from metamorf.constants import *
from metamorf.tools.log import Log

class ConfigValidator:

    def __init__(self,properties_file: dict, configuration_file: dict, log: Log):
        self.modules = []
        self.databases = []
        self.properties_file = properties_file
        self.configuration_file = configuration_file
        self.log = log
        self.configvalidator_name = 'Configuration Validation'

    def validate(self):
        result_properties_file = self.validate_properties_file()
        result_configuration_file = self.validate_configuration_file()
        return result_configuration_file and result_properties_file

    def validate_properties_file(self):
        """Validate properties file. Get some functions that are used to validate configuration file."""
        #Options
        if 'options' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - [Options]: options section is mandatory.", LOG_LEVEL_ERROR)
            return False
        if 'log_print' not in self.properties_file['options']:
            self.log.log(self.configvalidator_name, 'PropertiesFile - Options -> Missing Value: log_print', LOG_LEVEL_ERROR)
            return False

        #Commands
        if 'commands' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - [Commands]: arguments section is mandatory.", LOG_LEVEL_ERROR)
            return False
        for com in self.properties_file['commands']:
            if 'name' not in com:
                self.log.log(self.configvalidator_name, "PropertiesFile - [Commands]: There are a command without name attribute", LOG_LEVEL_ERROR)
                return False
            if 'description' not in com:
                self.log.log(self.configvalidator_name, "PropertiesFile - [Commands]: There are a command without description attribute", LOG_LEVEL_ERROR)
                return False
            if 'args' in com:
                for arg in com['args']:
                    if 'name' not in arg or 'options' not in arg or 'values' not in arg:
                        self.log.log(self.configvalidator_name, "PropertiesFile - [Commands]: arguments on command ["+com['name']+"] need to have [name], [options] and [values].", LOG_LEVEL_ERROR)
                        return False

        #Properties
        if 'name' not in self.properties_file :
            self.log.log(self.configvalidator_name,"PropertiesFile - Missing Value: name", LOG_LEVEL_ERROR)
            return False
        if 'description' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - Missing Value: description", LOG_LEVEL_ERROR)
            return False
        if 'version' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - Missing Value: version", LOG_LEVEL_ERROR)
            return False
        if 'author' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - Missing Value: author", LOG_LEVEL_ERROR)
            return False
        if 'contact' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - Missing Value: contact", LOG_LEVEL_ERROR)
            return False

        # Modules
        if 'modules' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - [Modules]: modules section is mandatory.", LOG_LEVEL_ERROR)
            return False

        for m in self.properties_file['modules']:
            module = m['name']
            if module not in self.modules:
                self.modules.append(module)
            else:
                self.log.log(self.configvalidator_name, "PropertiesFile - Module: ' + module + ' -> Module Duplicate", LOG_LEVEL_ERROR)
                return False
            if 'version' not in m:
                self.log.log(self.configvalidator_name, "PropertiesFile - Module: ' + module + ' -> Missing Value: version", LOG_LEVEL_ERROR)
                return False
            if 'functions' not in m:
                self.log.log(self.configvalidator_name, "PropertiesFile - Module: ' + module + ' -> Missing Value: functions", LOG_LEVEL_ERROR)
                return False

            for x in m['functions']:
                if 'name' not in x:
                    self.log.log(self.configvalidator_name, 'PropertiesFile - Module: ' + module + ' - Function ' + str(x) + '-> Missing Value: name', LOG_LEVEL_ERROR)
                    return False
                if 'values' not in x:
                    self.log.log(self.configvalidator_name, 'PropertiesFile - Module: ' + module + ' - Function ' + str(x) + '-> Missing Value: values', LOG_LEVEL_ERROR)
                    return False

        #Metadata and Data
        if 'database' not in self.properties_file:
            self.log.log(self.configvalidator_name, "PropertiesFile - [Database]: database section is mandatory.", LOG_LEVEL_ERROR)
            return False

        for m in self.properties_file['database']:
            if 'name' not in m:
                self.log.log(self.configvalidator_name, "PropertiesFile - Database: There are a database without name attribute", LOG_LEVEL_ERROR)
                return False
            if 'mandatory_fields' not in m:
                self.log.log(self.configvalidator_name, "PropertiesFile - Database: There are a database without mandatory_fields attribute", LOG_LEVEL_ERROR)
                return False
            if m['name'] not in self.databases:
                self.databases.append(m['name'])
        return True

    def validate_configuration_file(self):
        if self.configuration_file is None:
            return

        #Properties
        if 'name' not in self.configuration_file:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Missing Value: name", LOG_LEVEL_ERROR)
            return False
        if self.configuration_file['name'] is None:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Missing Value: name", LOG_LEVEL_ERROR)
            return False

        if 'owner' not in self.configuration_file:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Missing Value: owner", LOG_LEVEL_CRITICAL)
            return False
        if self.configuration_file['owner'] is None:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Missing Value: owner", LOG_LEVEL_CRITICAL)
            return False

        if 'output' not in self.configuration_file:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Missing Value: output", LOG_LEVEL_CRITICAL)
            return False

        # Validation OUTPUT get_type_options_for_output
        if 'type' not in self.configuration_file['output']:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Output: The attribute [type] needs to be indicated", LOG_LEVEL_CRITICAL)
            return False

        if self.configuration_file['output']['type'] not in self.get_type_options_for_output():
            self.log.log(self.configvalidator_name, 'ConfigurationFile - Output: The attribute [output][type] has an impossible value', LOG_LEVEL_CRITICAL)
            return False
        if self.configuration_file['output']['type'] is None:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Output: The attribute [output][type] needs to be indicated", LOG_LEVEL_CRITICAL)
            return False

        # Validation MODULES
        if 'modules' not in self.configuration_file:
            self.log.log(self.configvalidator_name, 'Configuration File - Modules: [modules] section is mandatory on the Configuration File.', LOG_LEVEL_ERROR)
            return False

        try:
            for m in self.configuration_file['modules']:
                if 'name' not in m:
                    self.log.log(self.configvalidator_name, 'ConfigurationFile - Modules: Need [name] attribute', LOG_LEVEL_ERROR)
                    return False
                if m['name'] not in self.modules:
                    self.log.log(self.configvalidator_name, 'ConfigurationFile - Module: ' + m['name'] + ' -> Doesn\'t exist: ' + m['name'], LOG_LEVEL_ERROR)
                    return False
                functions = self.get_functions_name_from_module(m['name'])
                for key,value in m.items():
                    if key=='name': continue
                    if key not in functions:
                        self.log.log(self.configvalidator_name, 'ConfigurationFile - Module: ' + m['name'] + ' -> Function: '+ key + ' doesn\'t exist', LOG_LEVEL_ERROR)
                        return False
                    values = self.get_options_from_function_and_module(m['name'], key)
                    if value not in values:
                        self.log.log(self.configvalidator_name, 'ConfigurationFile - Module: ' + m['name'] + ' -> Function: [' + key + '] has an impossible value', LOG_LEVEL_ERROR)
                        return False
        except Exception as e:
            self.log.log(self.configvalidator_name, "ConfigurationFile - Module: " + str(e), LOG_LEVEL_ERROR)


        # Validation Metadata
        if 'metadata' not in self.configuration_file:
            self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: [metadata] section is mandatory on the Configuration File.', LOG_LEVEL_ERROR)
            return False

        metadata_information = self.configuration_file['metadata']
        if 'connection_type' not in metadata_information:
            self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: The attribute [connection_type] needs to be indicated', LOG_LEVEL_ERROR)
            return False
        if metadata_information['connection_type'] is None:
            self.log.log(self.configvalidator_name, "[ERROR] ConfigurationFile - Metadata: The attribute [connection_type] needs to be indicated", LOG_LEVEL_CRITICAL)
            return False
        mandatory_fields = self.get_mandatory_fields_from_database(metadata_information['connection_type'])
            #First validation: All the fields are OK
        for key,value in metadata_information.items():
            if key=='connection_type': continue
            if key not in mandatory_fields:
                self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: Field ' + key + ' is not an option for this database.', LOG_LEVEL_ERROR)
                return False
            #Second validation: There are all the fields indicated
        for key,value in metadata_information.items():
            if key=='connection_type': continue
            if value == None:
                self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: Field ' + key + ' has not a value', LOG_LEVEL_ERROR)
                return False
            mandatory_fields.remove(key)
        if len(mandatory_fields)>0:
            self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: The following attributes needs to be indicated to permit the connection: ' + str(mandatory_fields), LOG_LEVEL_ERROR)
            return False

        # Validation Data
        if 'data' not in self.configuration_file:
            self.log.log(self.configvalidator_name, 'ConfigurationFile - Data: [data] section is mandatory on the Configuration File.', LOG_LEVEL_ERROR)
            return False

        data_information = self.configuration_file['data']
        if 'connection_type' not in data_information:
            self.log.log(self.configvalidator_name, '[ERROR] ConfigurationFile - Data: The attribute [connection_type] needs to be indicated', LOG_LEVEL_ERROR)
            return False
        if data_information['connection_type'] is None:
            self.log.log(self.configvalidator_name, "[ERROR] ConfigurationFile - Data: The attribute [connection_type] needs to be indicated", LOG_LEVEL_CRITICAL)
            return False

        mandatory_fields = self.get_mandatory_fields_from_database(data_information['connection_type'])
        # First validation: All the fields are OK
        for key, value in data_information.items():
            if key == 'connection_type': continue
            if key not in mandatory_fields:
                self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: Field ' + key + ' is not an option for this database.', LOG_LEVEL_ERROR)
                return False
            # Second validation: There are all the fields indicated
        for key, value in data_information.items():
            if key == 'connection_type': continue
            mandatory_fields.remove(key)
        if len(mandatory_fields) > 0:
            self.log.log(self.configvalidator_name, 'ConfigurationFile - Metadata: The following attributes needs to be indicated to permit the connection: ' + str(mandatory_fields), LOG_LEVEL_ERROR)
            return False

        return True


    def get_functions_name_from_module(self, module_name: str):
        functions = []
        for m in self.properties_file['modules']:
            if m['name'] == module_name:
                for x in m['functions']:
                    functions.append(x['name'])
        return functions

    def get_options_from_function_and_module(self, module_name: str, function_name: str):
        values = []
        for m in self.properties_file['modules']:
            if m['name'] == module_name:
                for x in m['functions']:
                    if x['name'] == function_name:
                        values = x['values']
        return values

    def get_mandatory_fields_from_database(self, database_name: str):
        result = []
        for m in self.properties_file['database']:
            if m['name'] == database_name:
                for x in m['mandatory_fields']:
                    result.append(x)
                return result
        raise ValueError('[ERROR] ConfigurationFile - The connection used: ['+ database_name + "] is not correct.")
        return None

    def it_exists_output_config(self, output_type: str, option_name: str, option_value: str):
        for x in self.properties_file['output']:
            if x['name'] == output_type:
                for y in x['options']:
                    if y['name'] == option_name and (option_value in y['values'] or y['values'] == []):
                        return True
        return False

    def get_type_options_for_output(self):
        return self.properties_file['output']['type']




from metamorf.constants import *
from metamorf.tools.log import Log

class MetadataValidator:

    def __init__(self, metadata, source_connection, configuration_file, log):
        self.metadata = metadata
        self.log = log
        self.validator_name = 'Metadata Validator'
        self.configuration_file = configuration_file
        self.connection = source_connection

        self.initialize_references()
        '''
        COMO OBTENER las columnas de una tabla
        variable_connection = self.configuration_file['data']['connection_type'].lower() + "_database"
        configuration_connection_data = self.configuration_file['data']
        configuration_connection_data[variable_connection] = (self.metadata_to_load.get_path_from_id_path(id_path)).database_name
        self.connection.setup_connection(configuration_connection_data, self.log)
        table_definition = self.connection.get_table_columns_definition(entity.table_name)'''

    def initialize_references(self):
        self.all_entity_type = []
        for x in self.metadata.om_ref_entity_type: self.all_entity_type.append(x.entity_type_name)
        self.all_join_type = []
        for x in self.metadata.om_ref_join_type: self.all_join_type.append(x.join_name)
        self.all_key_type = []
        for x in self.metadata.om_ref_key_type: self.all_key_type.append(x.key_type_name)
        self.all_order_type = []
        for x in self.metadata.om_ref_order_type: self.all_order_type.append(x.order_type_name)
        self.all_query_type = []
        for x in self.metadata.om_ref_query_type: self.all_query_type.append(x.query_type_name)

        self.all_cod_path = []
        self.all_cod_entity_elt = []

    def validate_metadata(self):
        self.log.log(self.validator_name, 'Starting Metadata Entry Validation', LOG_LEVEL_INFO)
        result = self.validate_metadata_entry_path()
        result = result and self.validate_metadata_entry_entity()
        result = result and self.validate_metadata_entry_dataset_mappings()




        self.log.log(self.validator_name, 'Metadata Entry Validation finished', LOG_LEVEL_INFO)
        return result

    def validate_metadata_entry_path(self):
        table_validation = 'ENTRY_PATH'
        self.log.log(self.validator_name, 'Start metadata validation on [ '+table_validation+' ]', LOG_LEVEL_DEBUG)
        result = True
        special_characters = '!@#$%^&*()-+?=,<>/.'
        for path in self.metadata.entry_path:
            if path.cod_path not in self.all_cod_path: self.all_cod_path.append(path.cod_path)
            if path.cod_path is None or path.cod_path == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COD_PATH ] can not be null', LOG_LEVEL_ERROR)
            if path.owner is None or path.owner == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ OWNER ] can not be null', LOG_LEVEL_ERROR)
            if any(c in special_characters for c in path.database_name):
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ ' + table_validation + ' ] -> [ DATABASE_NAME ] only accepts alphanumeric characters', LOG_LEVEL_ERROR)
            if any(c in special_characters for c in str(path.schema_name)):
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ ' + table_validation + ' ] -> [ SCHEMA_NAME ] only accepts alphanumeric characters', LOG_LEVEL_ERROR)

        self.log.log(self.validator_name, 'Finished metadata validation on [ '+table_validation+' ]', LOG_LEVEL_DEBUG)
        return result

    def validate_metadata_entry_entity(self):
        table_validation = 'ENTRY_ENTITY'
        self.log.log(self.validator_name, 'Start metadata validation on [ ' + table_validation + ' ]', LOG_LEVEL_DEBUG)
        result = True

        for entity in self.metadata.entry_entity:
            if entity.cod_entity not in self.all_cod_entity_elt: self.all_cod_entity_elt.append(entity.cod_entity)
            if entity.cod_entity is None or entity.cod_entity == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COD_ENTITY ] can not be null', LOG_LEVEL_ERROR)
            if entity.table_name is None or entity.table_name == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ TABLE_NAME ] can not be null', LOG_LEVEL_ERROR)
            if entity.entity_type is None or entity.entity_type == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ ENTITY_TYPE ] can not be null', LOG_LEVEL_ERROR)
            if entity.entity_type not in self.all_entity_type:
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ ENTITY_TYPE ] invalid value', LOG_LEVEL_ERROR)
            if entity.cod_path is None or entity.cod_path == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COD_PATH ] can not be null', LOG_LEVEL_ERROR)
            if entity.cod_path not in self.all_cod_path:
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ ' + table_validation + ' ] -> [ COD_PATH ] invalid value', LOG_LEVEL_ERROR)
            if entity.strategy not in self.all_query_type and entity.strategy != '' and entity.strategy is not None:
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COD_STRATEGY ] invalid value', LOG_LEVEL_ERROR)
            if entity.owner is None or entity.owner == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ OWNER ] can not be null', LOG_LEVEL_ERROR)

        self.log.log(self.validator_name, 'Finished metadata validation on [ ' + table_validation + ' ]', LOG_LEVEL_DEBUG)
        return result

    def validate_metadata_entry_dataset_mappings(self):
        table_validation = 'ENTRY_DATASET_MAPPINGS'
        self.log.log(self.validator_name, 'Start metadata validation on [ ' + table_validation + ' ]', LOG_LEVEL_DEBUG)
        result = True

        all_target_entity = []

        for entry in self.metadata.entry_dataset_mappings:
            if entry.cod_entity_source not in self.all_cod_entity_elt and entry.cod_entity_source is not None and entry.cod_entity_source != '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COD_ENTITY_SOURCE ] invalid value', LOG_LEVEL_ERROR)
            if entry.value_source is None or entry.value_source == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ VALUE_SOURCE ] can not be null', LOG_LEVEL_ERROR)
            if entry.cod_entity_target not in self.all_cod_entity_elt and entry.cod_entity_target is not None and entry.cod_entity_target != '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COD_ENTITY_TARGET ] invalid value', LOG_LEVEL_ERROR)
            if entry.column_name_target is None or entry.column_name_target == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COLUMN_NAME_TARGET ] can not be null', LOG_LEVEL_ERROR)
            if entry.column_type_target is None or entry.column_type_target == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COLUMN_TYPE_TARGET ] can not be null', LOG_LEVEL_ERROR)
            if entry.ordinal_position is None or entry.ordinal_position == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ ORDINAL_POSITION ] can not be null', LOG_LEVEL_ERROR)
            if entry.column_length is None or entry.column_length == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COLUMN_LENGTH ] can not be null', LOG_LEVEL_ERROR)
            if entry.column_precision is None or entry.column_precision == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ COLUMN_PRECISION ] can not be null', LOG_LEVEL_ERROR)
            if entry.num_branch is None or entry.num_branch == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ NUM_BRANCH ] can not be null', LOG_LEVEL_ERROR)
            if entry.key_type not in self.all_key_type and entry.key_type is not None and entry.key_type != '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ KEY_TYPE ] invalid value', LOG_LEVEL_ERROR)
            if entry.sw_distinct is None or entry.sw_distinct == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ SW_DISTINCT ] can not be null', LOG_LEVEL_ERROR)
            if entry.owner is None or entry.owner == '':
                result = False
                self.log.log(self.validator_name, 'Metadata Entry [ '+table_validation+' ] -> [ OWNER ] can not be null', LOG_LEVEL_ERROR)

            if entry.cod_entity_target not in all_target_entity: all_target_entity.append(entry.cod_entity_target)

        self.log.log(self.validator_name, 'Finished metadata validation on [ ' + table_validation + ' ]', LOG_LEVEL_DEBUG)
        return result

    def validate_metadata_x(self):
        table_validation = 'ENTRY_PATH'
        self.log.log(self.validator_name, 'Start metadata validation on [ ' + table_validation + ' ]', LOG_LEVEL_DEBUG)
        result = True

        for path in self.metadata.X:
            pass

        self.log.log(self.validator_name, 'Start metadata validation on [ ' + table_validation + ' ]', LOG_LEVEL_DEBUG)
        return result





from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *

class EngineBackup(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Backup"
        self.engine_command = "backup"

    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()

        self.metadata_actual = self.load_metadata(load_om=True, load_entry=False, load_ref=True, load_im=False, owner=self.owner)

        self.log.log(self.engine_name, "Start to backup the system for the owner ["+self.owner + "]", LOG_LEVEL_INFO)
        # OM_DATASET
        content = []
        content.append(COLUMNS_OM_DATASET)
        for registry in self.metadata_actual.om_dataset: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_DV
        content = []
        content.append(COLUMNS_OM_DATASET_DV)
        for registry in self.metadata_actual.om_dataset_dv: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_DV)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_T_ORDER
        content = []
        content.append(COLUMNS_OM_DATASET_T_ORDER)
        for registry in self.metadata_actual.om_dataset_t_order: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_T_ORDER)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_SPECIFICATION
        content = []
        content.append(COLUMNS_OM_DATASET_SPECIFICATION)
        for registry in self.metadata_actual.om_dataset_specification: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_SPECIFICATION)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_T_MAPPING
        content = []
        content.append(COLUMNS_OM_DATASET_T_MAPPING)
        for registry in self.metadata_actual.om_dataset_t_mapping: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_T_MAPPING)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_T_HAVING
        content = []
        content.append(COLUMNS_OM_DATASET_T_HAVING)
        for registry in self.metadata_actual.om_dataset_t_having: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_T_HAVING)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_T_AGG
        content = []
        content.append(COLUMNS_OM_DATASET_T_AGG)
        for registry in self.metadata_actual.om_dataset_t_agg: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_T_AGG)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_T_FILTER
        content = []
        content.append(COLUMNS_OM_DATASET_T_FILTER)
        for registry in self.metadata_actual.om_dataset_t_filter: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_T_FILTER)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_RELATIONSHIPS
        content = []
        content.append(COLUMNS_OM_DATASET_RELATIONSHIPS)
        for registry in self.metadata_actual.om_dataset_relationships: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_RELATIONSHIPS)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_EXECUTION
        content = []
        content.append(COLUMNS_OM_DATASET_EXECUTION)
        for registry in self.metadata_actual.om_dataset_execution: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_EXECUTION)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_PATH
        content = []
        content.append(COLUMNS_OM_DATASET_PATH)
        for registry in self.metadata_actual.om_dataset_path: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_PATH)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_DATASET_T_DISTINCT
        content = []
        content.append(COLUMNS_OM_DATASET_T_DISTINCT)
        for registry in self.metadata_actual.om_dataset_t_distinct: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_DATASET_T_DISTINCT)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        # OM_PROPERTIES
        content = []
        content.append(COLUMNS_OM_PROPERTIES)
        for registry in self.metadata_actual.om_properties: content.append(registry.get())
        file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_controller_final.set_file_location(os.path.join(ACTUAL_PATH , BACKUP_FILES_PATH), TABLE_OM_PROPERTIES)
        file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller_final.write_file(content)
        file_controller_final.close()

        self.log.log(self.engine_name, "Finish to backup the system for the owner [" + self.owner + "]", LOG_LEVEL_INFO)

        super().finish_execution()


from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
import os

class EngineDeploy(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Deploy"
        self.engine_command = "deploy"


    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()

        # ARCHITECTURE INSTALATION
        connection_type = self.configuration_file['metadata']['connection_type']
        # Get File that will be executed
        file_controller_init_sql = FileControllerFactory().get_file_reader(FILE_TYPE_SQL)
        file_controller_init_sql.set_file_location(os.path.join(PACKAGE_PATH , INITIALIZATION_FILE_PATH , connection_type.lower()), "pre_init_" + connection_type.lower() + ".sql")
        file_init_sql = file_controller_init_sql.read_file()

        # Get Connection and Execute the Initialization
        self.log.log(self.engine_name, "Deploying Metamorf Architecture on the Metadata Database", LOG_LEVEL_INFO)
        connection = ConnectionFactory().get_connection(connection_type)
        connection.setup_connection(self.configuration_file['metadata'], self.log)
        connection.execute(file_init_sql)
        connection.commit()
        connection.close()

        # METAMORF INSTALATION
        connection_type = self.configuration_file['metadata']['connection_type']
        # Get File that will be executed
        file_controller_init_sql = FileControllerFactory().get_file_reader(FILE_TYPE_SQL)
        file_controller_init_sql.set_file_location(os.path.join(PACKAGE_PATH, INITIALIZATION_FILE_PATH, connection_type.lower()), "init_" + connection_type.lower() + ".sql")
        file_init_sql = file_controller_init_sql.read_file()

        # Get Connection and Execute the Initialization
        self.log.log(self.engine_name, "Deploying Metamorf on the Database indicated on Configuration File", LOG_LEVEL_INFO)
        connection = ConnectionFactory().get_connection(connection_type)
        connection.setup_connection(self.configuration_file['metadata'], self.log)
        connection.execute(file_init_sql)
        connection.commit()
        connection.close()
        super().finish_execution()





import concurrent.futures

from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
from metamorf.tools.utils import get_list_nodes_from_metadata, get_node_with_with_query_execution_settings
import threading

class EngineRun(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Run"
        self.engine_command = "run"

    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()

        self.metadata = self.load_metadata(load_om=True, load_ref=True, load_entry=False, load_im=False, owner=self.owner)
        #TODO: Actually it executes all the project of the owner -> "all"
        nodes = get_list_nodes_from_metadata(self.metadata, self.log, "all")

        connection_type = self.configuration_file['metadata']['connection_type']
        connection = ConnectionFactory().get_connection(connection_type)

        process_finished_ok = []
        process_finished_nok = []
        for dataset in self.metadata.om_dataset:
            if dataset.id_entity_type == self.metadata.get_entity_type_from_entity_type_name(ENTITY_SRC).id_entity_type:
                process_finished_ok.append(dataset.dataset_name)

        self.log.log(self.engine_name, "Start the execution", LOG_LEVEL_INFO)
        index = 0
        results_nodes = []
        while nodes:
            node = nodes[index]
            if(set(node.predecessors).issubset(process_finished_ok)):
                node.init_execution()
                node.status = NODE_STATUS_RUNNING
                result_execution = self.execute_node(node, connection)
                if result_execution:
                    node.status = NODE_STATUS_FINISHED_OK
                    process_finished_ok.append(node.name)

                else:
                    node.status = NODE_STATUS_FINISHED_NOK
                    process_finished_nok.append(node.name)

                node.finish_execution()
                results_nodes.append(node)
                nodes.remove(node)

            elif(set(node.predecessors).issubset(process_finished_ok + process_finished_nok)):
                node.status = NODE_STATUS_SKIP
                process_finished_nok.append(node.name)
                results_nodes.append(node)
                nodes.remove(node)

            index += 1
            if index>=len(nodes): index = 0

        self.log.log(self.engine_name, "Finish the execution", LOG_LEVEL_INFO)
        if len(self.metadata.om_dataset)==0:
            self.log.log(self.engine_name, "There is no nodes to execute", LOG_LEVEL_WARNING)
        else: self.show_results(results_nodes)
        super().finish_execution()

    def show_results(self, nodes):
        self.log.log("RESULT EXECUTION", "**********************************************", LOG_LEVEL_INFO)
        self.log.log("RESULT EXECUTION", "Summary of the execution", LOG_LEVEL_INFO)
        for node in nodes:
            if node.status == NODE_STATUS_FINISHED_OK:
                self.log.log("RESULT EXECUTION", node.name + " --> Finished Ok in [" + "{:.3f}".format(node.time) + " seconds]", LOG_LEVEL_OK)
            if node.status == NODE_STATUS_FINISHED_NOK:
                self.log.log("RESULT EXECUTION", node.name + " --> Finished Not Ok in [" + "{:.3f}".format(node.time) + " seconds]", LOG_LEVEL_ERROR)
            if node.status == NODE_STATUS_SKIP:
                self.log.log("RESULT EXECUTION", node.name + " --> Skipped because a predecessor finished not ok", LOG_LEVEL_WARNING)
        self.log.log("RESULT EXECUTION", "**********************************************", LOG_LEVEL_INFO)

    def execute_node(self, node, connection):
        self.log.log(self.engine_name, "Start executing " + node.name, LOG_LEVEL_INFO)

        path = self.metadata.get_path_from_dataset_name(node.name)
        connection_type = self.configuration_file['metadata']['connection_type']
        configuration = self.configuration_file['data']
        configuration[connection_type + "_database"] = path.database_name
        configuration[connection_type + "_schema"] = path.schema_name
        connection.setup_connection(configuration,self.log)

        # Select Strategy of Materialization
        node = get_node_with_with_query_execution_settings(connection, self.metadata, node)

        result_execution = connection.execute(str(node.query))
        connection.commit()

        self.log.log(self.engine_name, "Finished " + node.name, LOG_LEVEL_INFO)

        return result_execution


from metamorf.engines.engine import Engine
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
from metamorf.tools.metadata_validator import MetadataValidator

class EngineMetadata(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Metadata"
        self.engine_command = "metadata"

    def run(self):
        super().start_execution()

        self.connection = ConnectionFactory().get_connection(self.configuration_file['data']['connection_type'])
        self.connection_metadata = ConnectionFactory().get_connection(self.configuration_file['metadata']['connection_type'])
        self.metadata_actual = self.load_metadata(load_om=True, load_entry=True, load_ref=True, load_im=False, owner=self.owner)

        # If there's nothing to load it finishes the execution
        if len(self.metadata_actual.entry_entity) == 0:
            self.log.log(self.engine_name, "There is nothing to validate.", LOG_LEVEL_WARNING)
            super().finish_execution(True)
            return

        metadata_validator = MetadataValidator(self.metadata_actual, self.connection, self.configuration_file, self.log)
        result_validation_metadata_entry = metadata_validator.validate_metadata()
        if result_validation_metadata_entry == False:
            self.log.log(self.engine_name, "Metadata Entry Validation has not passed. Please review the errors.", LOG_LEVEL_CRITICAL)
            super().finish_execution(False)
            return
        else:
            self.log.log(self.engine_name, "Metadata Entry validation - Ok", LOG_LEVEL_OK)

        super().finish_execution()


from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
from metamorf.tools.metadata import Metadata
from metamorf.tools.query import Query
import os

class EngineRestore(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Restore"
        self.engine_command = "restore"

    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()
        connection_type = self.configuration_file['metadata']['connection_type']
        self.connection = ConnectionFactory().get_connection(connection_type)
        self.connection.setup_connection(self.configuration_file['metadata'], self.log)

        result_delete = self.delete_all_entry_from_owner()
        if not result_delete:
            self.connection.rollback()
            super().finish_execution(result_delete)
        else:
            result = self.load_files_to_metadata_database()
            if not result: self.connection.rollback()
            else: self.connection.commit()
            super().finish_execution(result)

    def delete_all_entry_from_owner(self):
        result = True
        self.log.log(self.engine_name, "Starting deleting Metadata on database" , LOG_LEVEL_INFO)
        connection_type = self.configuration_file['metadata']['connection_type']
        metadata_tables = [TABLE_OM_DATASET, TABLE_OM_DATASET_EXECUTION, TABLE_OM_DATASET_T_ORDER,
                           TABLE_OM_DATASET_T_AGG, TABLE_OM_DATASET_T_DISTINCT, TABLE_OM_DATASET_SPECIFICATION,
                           TABLE_OM_DATASET_RELATIONSHIPS, TABLE_OM_DATASET_T_MAPPING, TABLE_OM_DATASET_T_FILTER,
                           TABLE_OM_DATASET_T_HAVING, TABLE_OM_DATASET_PATH , TABLE_OM_PROPERTIES, TABLE_OM_DATASET_DV]
        for file in metadata_tables:
            self.log.log(self.engine_name, "Deleting: " + file, LOG_LEVEL_INFO)
            query = Query()
            query.set_database(connection_type)
            query.set_target_table(file)
            query.set_is_truncate(True)
            res = self.connection.execute(str(query))
            result = result and res
        return result

    def refactor_file(self, file):
        for row in file:
            row[len(row)-2] = "'"+row[len(row)-2] +"'"
            if row[len(row)-1] is not None and row[len(row)-1] != '': row[len(row)-1] = "'"+ row[len(row)-1] + "'"
        return file

    def refactor_file_last_col(self, file):
        for row in file:
            row[len(row)-1] = "'"+row[len(row)-1] + "'"
        return file

    def load_files_to_metadata_database(self):
        self.log.log(self.engine_name, "Starting uploading Metadata Entry on database", LOG_LEVEL_INFO)
        result = True
        metadata = Metadata(self.log)
        connection_type = self.configuration_file['metadata']['connection_type']

        # TABLE_OM_DATASET
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH), TABLE_OM_DATASET + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET)
        query_values.set_insert_columns(COLUMNS_OM_DATASET)
        query_values.set_values(metadata.om_dataset)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_DV
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH), TABLE_OM_DATASET_DV + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_dv(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_DV)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_DV)
        query_values.set_values(metadata.om_dataset_dv)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_DV, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_EXECUTION
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH), TABLE_OM_DATASET_EXECUTION + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_execution(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_EXECUTION)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_EXECUTION)
        query_values.set_values(metadata.om_dataset_execution)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_EXECUTION, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_T_ORDER
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH), TABLE_OM_DATASET_T_ORDER + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_t_order(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_T_ORDER)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_T_ORDER)
        query_values.set_values(metadata.om_dataset_t_order)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_T_ORDER, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_T_AGG
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH), TABLE_OM_DATASET_T_AGG + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_t_agg(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_T_AGG)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_T_AGG)
        query_values.set_values(metadata.om_dataset_t_agg)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_T_AGG, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_T_DISTINCT
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_T_DISTINCT + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_t_distinct(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_T_DISTINCT)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_T_DISTINCT)
        query_values.set_values(metadata.om_dataset_t_distinct)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_T_DISTINCT, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_SPECIFICATION
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_SPECIFICATION + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_specification(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_SPECIFICATION)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_SPECIFICATION)
        query_values.set_values(metadata.om_dataset_specification)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_SPECIFICATION, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_RELATIONSHIPS
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_RELATIONSHIPS + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_relationships(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_RELATIONSHIPS)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_RELATIONSHIPS)
        query_values.set_values(metadata.om_dataset_relationships)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_RELATIONSHIPS, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_T_MAPPING
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_T_MAPPING + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_t_mapping(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_T_MAPPING)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_T_MAPPING)
        query_values.set_values(metadata.om_dataset_t_mapping)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_T_MAPPING, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_T_FILTER
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_T_FILTER + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_t_filter(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_T_FILTER)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_T_FILTER)
        query_values.set_values(metadata.om_dataset_t_filter)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_T_FILTER, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_T_HAVING
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_T_HAVING + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_t_having(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_T_HAVING)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_T_HAVING)
        query_values.set_values(metadata.om_dataset_t_having)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_T_HAVING, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_DATASET_PATH
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_DATASET_PATH + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_dataset_path(self.refactor_file(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_DATASET_PATH)
        query_values.set_insert_columns(COLUMNS_OM_DATASET_PATH)
        query_values.set_values(metadata.om_dataset_path)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_DATASET_PATH, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res

        # TABLE_OM_PROPERTIES
        file_reader = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
        file_reader.set_file_location(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH),
                                      TABLE_OM_PROPERTIES + "." + FILE_TYPE_CSV)
        file = file_reader.read_file()
        res = metadata.add_om_properties(self.refactor_file_last_col(file))
        result = result and res
        query_values = Query()
        query_values.set_database(connection_type)
        query_values.set_has_header(True)
        query_values.set_type(QUERY_TYPE_VALUES)
        query_values.set_target_table(TABLE_OM_PROPERTIES)
        query_values.set_insert_columns(COLUMNS_OM_PROPERTIES)
        query_values.set_values(metadata.om_properties)
        self.log.log(self.engine_name, "Loading: " + TABLE_OM_PROPERTIES, LOG_LEVEL_INFO)
        res = self.connection.execute(str(query_values))
        result = result and res



        return result



from datetime import datetime
from metamorf.constants import *
from metamorf.tools.filecontroller import FileControllerFactory

def generate_manifest(metadata, configuration, all_nodes):
    manifest = dict()

    file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_JSON)
    file_controller_final.set_file_location(ACTUAL_PATH, FILE_MANIFEST)
    file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)


    manifest_metadata = dict()
    manifest_metadata['metamorf_version_software'] = VERSION
    manifest_metadata['metamorf_version_metadata'] = metadata.get_version_from_metadata_database()
    manifest_metadata['generated_at'] = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    manifest_metadata['metadata_database'] = configuration['metadata']['connection_type']
    manifest_metadata['data_database'] = configuration['data']['connection_type']
    manifest_metadata['project_name'] = configuration['name']

    modules = dict()
    modules_elt = dict()
    modules_dv = dict()
    for module in configuration['modules']:
        if module['name'] == 'elt':
            modules_elt['status'] = module['status']
        if module['name'] == 'datavault':
            modules_dv['status'] = module['status']
            modules_dv['char_separator_naming'] = module['char_separator_naming']

    modules['elt'] = modules_elt
    modules['datavault'] = modules_dv

    data_database = configuration['data']
    metadata_database = configuration['metadata']

    nodes = []
    for node in all_nodes:
        new_node = dict()
        new_node['name'] = node.name
        new_node['predecessors'] = node.predecessors
        node.query.set_type(QUERY_TYPE_SELECT)
        new_node['query'] = str(node.query)
        nodes.append(new_node)

    datasets = []
    for dataset in metadata.om_dataset_information:
        d = dict()
        d_info = dict()
        d_info['owner'] = dataset.meta_owner
        d_info['entity_type'] = dataset.entity_type_full_name
        d_info['entity_type_description'] = dataset.entity_type_description
        d_info['module_name'] = dataset.module_name
        d_info['module_full_name'] = dataset.module_full_name
        d_info['module_description'] = dataset.module_description
        d_info['database_name'] = dataset.database_name
        d_info['schema_name'] = dataset.schema_name
        d_info['query_type'] = dataset.query_type_name
        d_info['query_type_description'] = dataset.query_type_description

        d_info['start_date'] = dataset.start_date

        d_relationships = []
        for rel in metadata.om_relationships:
            if rel.master_dataset_name == dataset.dataset_name or rel.detail_dataset_name == dataset.dataset_name:
                d_rel = dict()
                d_rel['master_dataset_name'] = rel.master_dataset_name
                d_rel['master_column_name'] = rel.master_column_name
                d_rel['detail_dataset_name'] = rel.detail_dataset_name
                d_rel['detail_column_name'] = rel.detail_column_name
                d_relationships.append(d_rel)
        d_info['relationships'] = d_relationships

        d_columns = []
        for col in metadata.om_dataset_specification_information:
            if col.dataset_name == dataset.dataset_name:
                d_col = dict()
                d_col['column_name'] = col.column_name
                d_col['column_type'] = col.column_type
                d_col['ordinal_position'] = col.ordinal_position
                d_col['is_nullable'] = col.is_nullable
                d_col['column_length'] = col.column_length
                d_col['column_precision'] = col.column_precision
                d_col['column_scale'] = col.column_scale
                d_columns.append(d_col)
        d_info['columns'] = d_columns

        d[dataset.dataset_name] = d_info
        datasets.append(d)

    manifest['metadata'] = manifest_metadata
    manifest['modules'] = modules
    manifest['data_database'] = data_database
    manifest['metadata_database'] = metadata_database
    manifest['nodes'] = nodes
    manifest['datasets'] = datasets

    file_controller_final.write_file(manifest)
    file_controller_final.close()
    return manifest


from metamorf.tools.query import *
import metamorf.tools.metadata
from metamorf.constants import *
'''
query = metamorf.tools.query.Query()
query.set_name_query('EJEMPLO')
query.set_is_with(False)
query.set_is_distinct(True)
query.set_select_columns(['ID', 'NAME'])
query.set_type(QUERY_TYPE_SELECT)
query.set_insert_columns(['ID', 'NAME'])
#query.setFromTables(['PRUEBA'])
query.set_from_tables_and_relations([metamorf.FromRelationQuery('A', 'ID', 'B', 'ID', 0, "="), metamorf.FromRelationQuery('D', 'ID', 'C', 'ID', 1, "=") , metamorf.tools.query.FromRelationQuery('B', 'ID', 'C', 'ID', 2, "=")])
query.set_where_filters(['ID > 0'])
#query.setHavingFilters(['count(1)>1'])
query.set_group_by_columns(['ID', 'NAME'])
query.set_order_by_columns([metamorf.OrderByQuery('ID', 'asc') , metamorf.OrderByQuery('NAME', 'desc')])

queryWithA = metamorf.tools.query.Query()
queryWithA.set_name_query("A")
queryWithA.set_is_with(True)
queryWithA.set_select_columns(['ID'])
queryWithA.set_from_tables(['Z'])

queryWithOrigen = metamorf.tools.query.Query()
queryWithOrigen.set_name_query("Z")
queryWithOrigen.set_is_with(True)
queryWithOrigen.set_select_columns(['ID_2 as ID', 'FECHA'])
queryWithOrigen.set_from_tables(['ORIGEN'])
queryWithOrigen.set_where_filters(['FECHA>SYSDATE'])

query.add_subquery(queryWithA)
query.add_subquery(queryWithOrigen)

queryUnion = metamorf.tools.query.Query()
queryUnion.set_name_query('EJEMPLO')
queryUnion.set_is_with(False)
queryUnion.set_is_distinct(True)
queryUnion.set_select_columns(['FUNCIONA.ID', 'FUNCIONA.NAME'])
queryUnion.set_type(QUERY_TYPE_SELECT)
queryUnion.set_insert_columns(['ID', 'NAME'])
queryUnion.set_from_tables(["FUNCIONA"])

query.add_unionquery(queryUnion)

#print(str(query))
'''

queryValues = metamorf.tools.query.Query()
queryValues.set_has_header(True)
queryValues.set_type(QUERY_TYPE_VALUES)
queryValues.set_name_query('ENTRY_AGGREGATORS')
queryValues.set_insert_columns(COLUMNS_ENTRY_AGGREGATORS)

metadata = metamorf.tools.metadata.Metadata()
metadata.add_entry_aggregators([['COD_ENTITY_TARGET', 'COD_ENTITY_SOURCE', 'COLUMN_NAME', 'NUM_BRANCH'], ['TARGET', 'SRC','ID','1'], ['TARGETO', 'SRC', 'NAME', '2']])

queryValues.set_values(metadata.entry_aggregators)

print(str(queryValues))








from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.constants import *
import yaml
import os

class EngineInit(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Init"
        self.engine_command = "init"
        self.database_default = CONNECTION_SQLITE.lower()

        self.metadata_database = None
        self.data_database = None
        if 'metadata' in self.arguments:
            self.metadata_database = self.arguments['metadata']
        if 'database' in self.arguments:
            self.data_database = self.arguments['database']
        if self.metadata_database is not None and self.data_database is not None:
            self.log.log(self.engine_name,
                         "The configuration file is going to be created for the data database [" + self.data_database + "] and metadata database ["+ self.metadata_database + "]",
                         LOG_LEVEL_INFO)
        if self.metadata_database is None and self.data_database is None:
            self.metadata_database = self.database_default
            self.data_database = self.database_default
            self.log.log(self.engine_name, "The configuration file is going to be created for the default database [" + self.database_default + "]", LOG_LEVEL_INFO)
        if self.metadata_database is None and self.data_database is not None:
            self.metadata_database = self.data_database
            self.log.log(self.engine_name,
                         "The configuration file is going to be created for the Data Database selected [" + self.data_database + "]",
                         LOG_LEVEL_INFO)
        if self.data_database is None and self.metadata_database is not None:
            self.log.log(self.engine_name,
                         "The configuration file is going to be created for the Metadata Database selected [" + self.metadata_database + "]",
                         LOG_LEVEL_INFO)
            self.data_database = self.metadata_database

    def run(self):
        super().start_execution(need_configuration_file=False)

        # Create Configuration File
        file_controller_configuration = FileControllerFactory().get_file_reader(FILE_TYPE_YML)
        file_controller_configuration.set_file_location(ACTUAL_PATH, CONFIGURATION_FILE_NAME)
        file_controller_configuration.setup_writer(FILE_WRITER_NEW_FILE)

        # Read Template OUTPUT & MODULES
        file_controller_configuration_template = FileControllerFactory().get_file_reader(FILE_TYPE_YML)
        file_controller_configuration_template.set_file_location(os.path.join(PACKAGE_PATH, CONFIGURATION_FILE_PATH), CONFIGURATION_FILE_NAME)
        self.configuration_file = file_controller_configuration_template.read_file()
        file_controller_configuration.write_file(self.configuration_file)

        # Read Template DATA
        file_controller_configuration_template = FileControllerFactory().get_file_reader(FILE_TYPE_YML)
        file_controller_configuration_template.set_file_location(os.path.join(PACKAGE_PATH, CONFIGURATION_FILE_PATH, self.data_database.lower()), CONFIGURATION_FILE_DATA + self.data_database + "." + FILE_TYPE_YML)
        self.configuration_file = file_controller_configuration_template.read_file()
        file_controller_configuration.write_file(self.configuration_file)

        # Read Template METADATA
        file_controller_configuration_template = FileControllerFactory().get_file_reader(FILE_TYPE_YML)
        file_controller_configuration_template.set_file_location(os.path.join(PACKAGE_PATH, CONFIGURATION_FILE_PATH, self.metadata_database.lower()), CONFIGURATION_FILE_METADATA + self.metadata_database + "." + FILE_TYPE_YML)
        self.configuration_file = file_controller_configuration_template.read_file()
        file_controller_configuration.write_file(self.configuration_file)

        # Create folder
        if not os.path.exists(os.path.join(ACTUAL_PATH, ENTRY_FILES_PATH)):
            os.mkdir(os.path.join(ACTUAL_PATH, ENTRY_FILES_PATH))
        if not os.path.exists(os.path.join(ACTUAL_PATH, OUTPUT_FILES_PATH)):
            os.mkdir(os.path.join(ACTUAL_PATH, OUTPUT_FILES_PATH))
        if not os.path.exists(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH)):
            os.mkdir(os.path.join(ACTUAL_PATH, BACKUP_FILES_PATH))

        # Create files - Metadata Entry
        all_entry_files = os.listdir(os.path.join(PACKAGE_PATH, ENTRY_FILES_PATH))
        for file in all_entry_files:
            file_controller = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
            file_controller.set_file_location(os.path.join(PACKAGE_PATH, ENTRY_FILES_PATH), file)
            file_to_write = file_controller.read_file()

            file_controller_final = FileControllerFactory().get_file_reader(FILE_TYPE_CSV)
            file_controller_final.set_file_location(os.path.join(ACTUAL_PATH, ENTRY_FILES_PATH), file)
            file_controller_final.setup_writer(FILE_WRITER_NEW_FILE)
            file_controller_final.write_file(file_to_write)

        super().finish_execution()


from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
from metamorf.tools.query import Query

class EngineCommit(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Commit"
        self.engine_command = "commit"

    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()

        self.log.log(self.engine_name, "Starting to commit metadata from owner ["+self.owner+"]", LOG_LEVEL_INFO)
        connection_type = self.configuration_file['metadata']['connection_type']
        self.connection = ConnectionFactory().get_connection(connection_type)
        self.connection.setup_connection(self.configuration_file['metadata'], self.log)

        self.delete_metadata_git()
        self.copy_metadata_git()

        self.connection.commit()
        self.connection.close()

        super().finish_execution()


    def delete_metadata_git(self):
        self.log.log(self.engine_name, "Starting to delete previous metadata version", LOG_LEVEL_INFO)

        where_filter = COLUMN_GIT_ENTRY_OWNER+"='"+self.owner+"'"
        connection_type = (self.configuration_file['metadata']['connection_type']).upper()

        # GIT_ENTRY_PATH
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_PATH)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_ENTITY
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_ENTITY)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_ORDER
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_ORDER)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_HAVING
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_HAVING)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_FILTERS
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_FILTERS)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_DATASET_MAPPINGS
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_DATASET_MAPPINGS)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_DATASET_RELATIONSHIPS
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_DATASET_RELATIONSHIPS)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_AGGREGATORS
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_AGGREGATORS)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_ENTITY
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_DV_ENTITY)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_MAPPINGS
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_DV_MAPPINGS)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_PROPERTIES
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_DV_PROPERTIES)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_PROPERTIES
        query = Query()
        query.set_database(connection_type)
        query.set_type(QUERY_TYPE_DELETE)
        query.set_target_table(TABLE_GIT_ENTRY_FILES)
        query.set_where_filters(where_filter)
        self.connection.execute(str(query))

        self.log.log(self.engine_name, "Finished to delete previous metadata version", LOG_LEVEL_INFO)

    def copy_metadata_git(self):
        self.log.log(self.engine_name, "Starting to load active metadata version", LOG_LEVEL_INFO)
        where_filter = COLUMN_ENTRY_OWNER + "='" + self.owner + "'"
        connection_type = (self.configuration_file['metadata']['connection_type']).upper()

        # GIT_ENTRY_PATH
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_PATH)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_PATH)
        query.set_select_columns(COLUMNS_ENTRY_PATH)
        query.set_from_tables([TABLE_ENTRY_PATH])
        self.connection.execute(str(query))

        # GIT_ENTRY_ENTITY
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_ENTITY)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_ENTITY)
        query.set_select_columns(COLUMNS_ENTRY_ENTITY)
        query.set_from_tables([TABLE_ENTRY_ENTITY])
        self.connection.execute(str(query))

        # GIT_ENTRY_ORDER
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_ORDER)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_ORDER)
        query.set_select_columns(COLUMNS_ENTRY_ORDER)
        query.set_from_tables([TABLE_ENTRY_ORDER])
        self.connection.execute(str(query))

        # GIT_ENTRY_HAVING
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_HAVING)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_HAVING)
        query.set_select_columns(COLUMNS_ENTRY_HAVING)
        query.set_from_tables([TABLE_ENTRY_HAVING])
        self.connection.execute(str(query))

        # GIT_ENTRY_FILTERS
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_FILTERS)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_FILTERS)
        query.set_select_columns(COLUMNS_ENTRY_FILTERS)
        query.set_from_tables([TABLE_ENTRY_FILTERS])
        self.connection.execute(str(query))

        # GIT_ENTRY_DATASET_MAPPINGS
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_DATASET_MAPPINGS)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_DATASET_MAPPINGS)
        query.set_select_columns(COLUMNS_ENTRY_DATASET_MAPPINGS)
        query.set_from_tables([TABLE_ENTRY_DATASET_MAPPINGS])
        self.connection.execute(str(query))

        # GIT_ENTRY_DATASET_RELATIONSHIPS
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_DATASET_RELATIONSHIPS)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_DATASET_RELATIONSHIPS)
        query.set_select_columns(COLUMNS_ENTRY_DATASET_RELATIONSHIPS)
        query.set_from_tables([TABLE_ENTRY_DATASET_RELATIONSHIPS])
        self.connection.execute(str(query))

        # GIT_ENTRY_AGGREGATORS
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_AGGREGATORS)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_AGGREGATORS)
        query.set_select_columns(COLUMNS_ENTRY_AGGREGATORS)
        query.set_from_tables([TABLE_ENTRY_AGGREGATORS])
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_ENTITY
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_DV_ENTITY)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_DV_ENTITY)
        query.set_select_columns(COLUMNS_ENTRY_DV_ENTITY)
        query.set_from_tables([TABLE_ENTRY_DV_ENTITY])
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_MAPPINGS
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_DV_MAPPINGS)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_DV_MAPPINGS)
        query.set_select_columns(COLUMNS_ENTRY_DV_MAPPINGS)
        query.set_from_tables([TABLE_ENTRY_DV_MAPPINGS])
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_PROPERTIES
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_DV_PROPERTIES)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_DV_PROPERTIES)
        query.set_select_columns(COLUMNS_ENTRY_DV_PROPERTIES)
        query.set_from_tables([TABLE_ENTRY_DV_PROPERTIES])
        self.connection.execute(str(query))

        # GIT_ENTRY_DV_PROPERTIES
        query = Query()
        query.set_type(QUERY_TYPE_INSERT)
        query.set_database(connection_type)
        query.set_where_filters(where_filter)
        query.set_target_table(TABLE_GIT_ENTRY_FILES)
        query.set_insert_columns(COLUMNS_GIT_ENTRY_FILES)
        query.set_select_columns(COLUMNS_ENTRY_FILES)
        query.set_from_tables([TABLE_ENTRY_FILES])
        self.connection.execute(str(query))


        self.log.log(self.engine_name, "Finished to load active metadata version", LOG_LEVEL_INFO)


from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
from metamorf.tools.utils import get_list_nodes_from_metadata, get_node_with_with_query_execution_settings
import re
import os

class EngineOutput(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Output"
        self.engine_command = "output"

    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()

        output_type = self.configuration_file['output']['type']

        self.metadata = self.load_metadata(load_om=True, load_ref=True, load_entry=False, load_im=False, owner=self.owner)

        all_nodes = get_list_nodes_from_metadata(self.metadata, self.log, "all")

        self.log.log(self.engine_name, "Start to Generate Files with [" + output_type + "] configuration", LOG_LEVEL_INFO)

        connection_type = self.configuration_file['metadata']['connection_type']
        connection = ConnectionFactory().get_connection(connection_type)

        for node in all_nodes:

            path = self.metadata.get_path_from_dataset_name(node.name)
            configuration = self.configuration_file['data']
            configuration[connection_type + "_database"] = path.database_name
            configuration[connection_type + "_schema"] = path.schema_name
            connection.setup_connection(configuration, self.log)

            # Select Strategy of Materialization
            if output_type.upper() == OUTPUT_TYPE_DBT.upper():
                node.query.set_type(QUERY_TYPE_SELECT)
                content = str(node.query)
                content = self.dbt_create_config_for_models(node) + "\n\n" + content
                all_substitutions = node.predecessors
                all_substitutions.append(node.name)
                for table in all_substitutions:
                    dataset = self.metadata.get_dataset_from_dataset_name(table)
                    fqdn = self.metadata.get_dataset_fqdn_from_dataset_name(table)
                    if dataset.id_entity_type == self.metadata.get_entity_type_from_entity_type_name(ENTITY_SRC).id_entity_type:
                        path = self.metadata.get_path_from_id_path(dataset.id_path)
                        content = re.sub("\s+"+fqdn+"\s+|\r", " {{source('"+self.get_source_dbt_name_from_path(path)+"','"+table+"')}} ", content )
                    elif table == node.name:
                        content = re.sub("\s+" + fqdn + "\s+|\r", " {{this}} ", content)
                    else:
                        content = re.sub("\s+" + fqdn + "\s|\r+", " {{ref('" + table + "')}} ", content)

            else:
                node = get_node_with_with_query_execution_settings(connection, self.metadata, node)
                content = str(node.query)


            file_controller = FileControllerFactory().get_file_reader(FILE_TYPE_SQL)
            file_controller.set_file_location(os.path.join(ACTUAL_PATH, OUTPUT_FILES_PATH), node.name)
            file_controller.setup_writer(FILE_WRITER_NEW_FILE)
            file_controller.write_file(content)


        # Create source files for DBT
        if output_type.upper() == OUTPUT_TYPE_DBT.upper():
            self.dbt_create_source_yml()

        self.log.log(self.engine_name, "Finish Generate Files", LOG_LEVEL_INFO)
        super().finish_execution()

    def get_source_dbt_name_from_path(self, path):
        return 'metamorf' + self.get_fqdn_from_path(path)

    def get_fqdn_from_path(self, path):
        fqdn = ""
        if path.database_name is None or path.database_name == "":
            if path.schema_name is not None and path.schema_name != "":
                fqdn = "_" + path.schema_name
        else:
            if path.schema_name is None or path.schema_name == "":
                fqdn = "_" + path.database_name
            else:
                fqdn = "_" + path.database_name + "_" + path.schema_name
        return fqdn

    def get_paths_with_sources(self):
        path_with_source = []
        for dataset in self.metadata.om_dataset:
            if dataset.end_date is not None: continue
            if dataset.id_entity_type == self.metadata.get_entity_type_from_entity_type_name(ENTITY_SRC).id_entity_type:
                possible_path = self.metadata.get_path_from_id_path(dataset.id_path)
                if possible_path not in path_with_source:
                    path_with_source.append(possible_path)
        return path_with_source

    def get_sources_from_path(self, path):
        sources = []
        for dataset in self.metadata.om_dataset:
            if dataset.end_date is not None: continue
            if dataset.id_path == path.id_path and dataset.id_entity_type == self.metadata.get_entity_type_from_entity_type_name(ENTITY_SRC).id_entity_type:
                sources.append(dataset.dataset_name)
        return sources

    def dbt_create_source_yml(self):
        paths_with_sources = self.get_paths_with_sources()
        # Get all Paths with Sources
        yml_content = dict()
        yml_content['version'] = 2
        yml_content['sources'] = []

        for path in paths_with_sources:
            yml_name = dict()
            yml_name['name'] = self.get_source_dbt_name_from_path(path)
            yml_name['description'] = "Source generated by Metamorf"
            if path.database_name is not None and path.database_name != '':
                yml_name['database'] = path.database_name
            if path.schema_name is not None and path.schema_name != '':
                yml_name['schema'] = path.schema_name
            yml_name['tables'] = []
            all_sources_from_path = self.get_sources_from_path(path)

            for src in all_sources_from_path:
                yml_table = dict()
                yml_table['name'] = src
                yml_name['tables'].append(yml_table)

            yml_content['sources'].append(yml_name)

        file_controller = FileControllerFactory().get_file_reader(FILE_TYPE_YML)
        file_controller.set_file_location(os.path.join(ACTUAL_PATH, OUTPUT_FILES_PATH), "sources_metamorf")
        file_controller.setup_writer(FILE_WRITER_NEW_FILE)
        file_controller.write_file(yml_content)

    def dbt_create_config_for_models(self, node):
        id_query_type = self.metadata.get_dataset_execution_from_id_dataset(self.metadata.get_dataset_from_dataset_name(node.name).id_dataset).id_query_type
        config = "{{ config(materialized='"
        #TODO: Add more options to config DBT
        if id_query_type == QUERY_TYPE_TRUNCATE_AND_INSERT: config += "table"
        if id_query_type == QUERY_TYPE_INSERT or id_query_type == QUERY_TYPE_UPDATE: config += "incremental'"
        if id_query_type == QUERY_TYPE_VIEW: config += "view'"
        if id_query_type == QUERY_TYPE_MERGE:
            all_pk = []
            dataset_id = self.metadata.get_dataset_from_dataset_name(node.name).id_dataset
            for dataset_spec in self.metadata.om_dataset_specification:
                if dataset_spec.end_date is not None: continue
                if dataset_spec.id_dataset == dataset_id and dataset_spec.id_key_type == self.metadata.get_key_type_from_key_type_name(KEY_TYPE_PRIMARY_KEY).id_key_type:
                    all_pk.append(dataset_spec.column_name)

            config += "incremental', unique_key=["
            for pk in all_pk:
                config += "'"+pk+"'" +","
            config = config[:-1]
            config+= "]"
        config += ")}}"
        return config



from metamorf.engines.engine import Engine
from metamorf.tools.filecontroller import FileControllerFactory
from metamorf.tools.connection import ConnectionFactory
from metamorf.constants import *
from metamorf.tools.query import Query

class EngineRecover(Engine):

    def _initialize_engine(self):
        self.engine_name = "Engine Recover"
        self.engine_command = "recover"

        self.tables_to_load = [TABLE_ENTRY_AGGREGATORS, TABLE_ENTRY_DATASET_MAPPINGS,
                               TABLE_ENTRY_DATASET_RELATIONSHIPS, TABLE_ENTRY_ENTITY, TABLE_ENTRY_FILTERS,
                               TABLE_ENTRY_ORDER, TABLE_ENTRY_PATH, TABLE_ENTRY_HAVING,
                               TABLE_ENTRY_DV_ENTITY, TABLE_ENTRY_DV_MAPPINGS, TABLE_ENTRY_DV_PROPERTIES,
                               TABLE_ENTRY_FILES]
        if 'select' in self.arguments:
            if self.arguments['select'] == "all" or self.arguments['select'] == "*":
                self.tables_to_load = [TABLE_ENTRY_AGGREGATORS, TABLE_ENTRY_DATASET_MAPPINGS,
                                       TABLE_ENTRY_DATASET_RELATIONSHIPS, TABLE_ENTRY_ENTITY, TABLE_ENTRY_FILTERS,
                                       TABLE_ENTRY_ORDER, TABLE_ENTRY_PATH, TABLE_ENTRY_HAVING,
                                       TABLE_ENTRY_DV_ENTITY, TABLE_ENTRY_DV_MAPPINGS, TABLE_ENTRY_DV_PROPERTIES,
                                       TABLE_ENTRY_FILES]
            if self.arguments['select'].lower() == TABLE_ENTRY_DATASET_MAPPINGS.lower():
                self.tables_to_load = [TABLE_ENTRY_DATASET_MAPPINGS]
            if self.arguments['select'].lower() == TABLE_ENTRY_DATASET_RELATIONSHIPS.lower():
                self.tables_to_load = [TABLE_ENTRY_DATASET_RELATIONSHIPS]
            if self.arguments['select'].lower() == TABLE_ENTRY_ENTITY.lower():
                self.tables_to_load = [TABLE_ENTRY_ENTITY]
            if self.arguments['select'].lower() == TABLE_ENTRY_FILTERS.lower():
                self.tables_to_load = [TABLE_ENTRY_FILTERS]
            if self.arguments['select'].lower() == TABLE_ENTRY_ORDER.lower():
                self.tables_to_load = [TABLE_ENTRY_ORDER]
            if self.arguments['select'].lower() == TABLE_ENTRY_PATH.lower():
                self.tables_to_load = [TABLE_ENTRY_PATH]
            if self.arguments['select'].lower() == TABLE_ENTRY_HAVING.lower():
                self.tables_to_load = [TABLE_ENTRY_HAVING]
            if self.arguments['select'].lower() == TABLE_ENTRY_AGGREGATORS.lower():
                self.tables_to_load = [TABLE_ENTRY_AGGREGATORS]
            if self.arguments['select'].lower() == TABLE_ENTRY_DV_ENTITY.lower():
                self.tables_to_load = [TABLE_ENTRY_DV_ENTITY]
            if self.arguments['select'].lower() == TABLE_ENTRY_DV_MAPPINGS.lower():
                self.tables_to_load = [TABLE_ENTRY_DV_MAPPINGS]
            if self.arguments['select'].lower() == TABLE_ENTRY_DV_PROPERTIES.lower():
                self.tables_to_load = [TABLE_ENTRY_DV_PROPERTIES]
            if self.arguments['select'].lower() == TABLE_ENTRY_FILES.lower():
                self.tables_to_load = [TABLE_ENTRY_FILES]

    def run(self):
        # Starts the execution loading the Configuration File. If there is an error it finishes the execution.
        super().start_execution()

        self.log.log(self.engine_name, "Starting to commit metadata from owner ["+self.owner+"]", LOG_LEVEL_INFO)
        connection_type = self.configuration_file['metadata']['connection_type']
        self.connection = ConnectionFactory().get_connection(connection_type)
        self.connection.setup_connection(self.configuration_file['metadata'], self.log)

        self.delete_metadata_entry()
        self.copy_metadata_git()

        self.connection.commit()
        self.connection.close()

        super().finish_execution()


    def delete_metadata_entry(self):
        self.log.log(self.engine_name, "Starting to delete actual metadata", LOG_LEVEL_INFO)

        where_filter = COLUMN_ENTRY_OWNER+"='"+self.owner+"'"
        connection_type = self.configuration_file['metadata']['connection_type']

        # ENTRY_PATH
        if TABLE_ENTRY_PATH in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_PATH)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_ENTITY
        if TABLE_ENTRY_ENTITY in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_ENTITY)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_ORDER
        if TABLE_ENTRY_ORDER in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_ORDER)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_HAVING
        if TABLE_ENTRY_HAVING in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_HAVING)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_FILTERS
        if TABLE_ENTRY_FILTERS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_FILTERS)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_DATASET_MAPPINGS
        if TABLE_ENTRY_DATASET_MAPPINGS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_DATASET_MAPPINGS)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_DATASET_RELATIONSHIPS
        if TABLE_ENTRY_DATASET_RELATIONSHIPS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_DATASET_RELATIONSHIPS)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_AGGREGATORS
        if TABLE_ENTRY_AGGREGATORS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_AGGREGATORS)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_DV_ENTITY
        if TABLE_ENTRY_DV_ENTITY in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_DV_ENTITY)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_DV_MAPPINGS
        if TABLE_ENTRY_DV_MAPPINGS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_DV_MAPPINGS)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_DV_PROPERTIES
        if TABLE_ENTRY_DV_PROPERTIES in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_DV_PROPERTIES)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        # ENTRY_FILES
        if TABLE_ENTRY_FILES in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_DELETE)
            query.set_target_table(TABLE_ENTRY_FILES)
            query.set_where_filters(where_filter)
            self.connection.execute(str(query))

        self.log.log(self.engine_name, "Finished to delete actual metadata", LOG_LEVEL_INFO)

    def copy_metadata_git(self):
        self.log.log(self.engine_name, "Starting to load backup metadata", LOG_LEVEL_INFO)
        where_filter = COLUMN_GIT_ENTRY_OWNER + "='" + self.owner + "'"
        connection_type = self.configuration_file['metadata']['connection_type']

        # GIT_ENTRY_PATH
        if TABLE_ENTRY_PATH in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_PATH)
            query.set_insert_columns(COLUMNS_ENTRY_PATH)
            query.set_select_columns(COLUMNS_GIT_ENTRY_PATH)
            query.set_from_tables([TABLE_GIT_ENTRY_PATH])
            self.connection.execute(str(query))

        # GIT_ENTRY_ENTITY
        if TABLE_ENTRY_ENTITY in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_ENTITY)
            query.set_insert_columns(COLUMNS_ENTRY_ENTITY)
            query.set_select_columns(COLUMNS_GIT_ENTRY_ENTITY)
            query.set_from_tables([TABLE_GIT_ENTRY_ENTITY])
            self.connection.execute(str(query))

        # GIT_ENTRY_ORDER
        if TABLE_ENTRY_ORDER in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_ORDER)
            query.set_insert_columns(COLUMNS_ENTRY_ORDER)
            query.set_select_columns(COLUMNS_GIT_ENTRY_ORDER)
            query.set_from_tables([TABLE_GIT_ENTRY_ORDER])
            self.connection.execute(str(query))

        # GIT_ENTRY_HAVING
        if TABLE_ENTRY_HAVING in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_HAVING)
            query.set_insert_columns(COLUMNS_ENTRY_HAVING)
            query.set_select_columns(COLUMNS_GIT_ENTRY_HAVING)
            query.set_from_tables([TABLE_GIT_ENTRY_HAVING])
            self.connection.execute(str(query))

        # GIT_ENTRY_FILTERS
        if TABLE_ENTRY_FILTERS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_FILTERS)
            query.set_insert_columns(COLUMNS_ENTRY_FILTERS)
            query.set_select_columns(COLUMNS_GIT_ENTRY_FILTERS)
            query.set_from_tables([TABLE_GIT_ENTRY_FILTERS])
            self.connection.execute(str(query))

        # GIT_ENTRY_DATASET_MAPPINGS
        if TABLE_ENTRY_DATASET_MAPPINGS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_DATASET_MAPPINGS)
            query.set_insert_columns(COLUMNS_ENTRY_DATASET_MAPPINGS)
            query.set_select_columns(COLUMNS_GIT_ENTRY_DATASET_MAPPINGS)
            query.set_from_tables([TABLE_GIT_ENTRY_DATASET_MAPPINGS])
            self.connection.execute(str(query))

        # GIT_ENTRY_DATASET_RELATIONSHIPS
        if TABLE_ENTRY_DATASET_RELATIONSHIPS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_DATASET_RELATIONSHIPS)
            query.set_insert_columns(COLUMNS_ENTRY_DATASET_RELATIONSHIPS)
            query.set_select_columns(COLUMNS_GIT_ENTRY_DATASET_RELATIONSHIPS)
            query.set_from_tables([TABLE_GIT_ENTRY_DATASET_RELATIONSHIPS])
            self.connection.execute(str(query))

        # GIT_ENTRY_AGGREGATORS
        if TABLE_ENTRY_AGGREGATORS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_AGGREGATORS)
            query.set_insert_columns(COLUMNS_ENTRY_AGGREGATORS)
            query.set_select_columns(COLUMNS_GIT_ENTRY_AGGREGATORS)
            query.set_from_tables([TABLE_GIT_ENTRY_AGGREGATORS])
            self.connection.execute(str(query))

        # GIT_ENTRY_DV_ENTITY
        if TABLE_ENTRY_DV_ENTITY in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_DV_ENTITY)
            query.set_insert_columns(COLUMNS_ENTRY_DV_ENTITY)
            query.set_select_columns(COLUMNS_GIT_ENTRY_DV_ENTITY)
            query.set_from_tables([TABLE_GIT_ENTRY_DV_ENTITY])
            self.connection.execute(str(query))

        # GIT_ENTRY_DV_MAPPINGS
        if TABLE_ENTRY_DV_MAPPINGS in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_DV_MAPPINGS)
            query.set_insert_columns(COLUMNS_ENTRY_DV_MAPPINGS)
            query.set_select_columns(COLUMNS_GIT_ENTRY_DV_MAPPINGS)
            query.set_from_tables([TABLE_GIT_ENTRY_DV_MAPPINGS])
            self.connection.execute(str(query))

        # GIT_ENTRY_DV_PROPERTIES
        if TABLE_ENTRY_DV_PROPERTIES in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_DV_PROPERTIES)
            query.set_insert_columns(COLUMNS_ENTRY_DV_PROPERTIES)
            query.set_select_columns(COLUMNS_GIT_ENTRY_DV_PROPERTIES)
            query.set_from_tables([TABLE_GIT_ENTRY_DV_PROPERTIES])
            self.connection.execute(str(query))

        # GIT_ENTRY_DV_PROPERTIES
        if TABLE_ENTRY_FILES in self.tables_to_load:
            query = Query()
            query.set_database(connection_type)
            query.set_type(QUERY_TYPE_INSERT)
            query.set_where_filters(where_filter)
            query.set_target_table(TABLE_ENTRY_FILES)
            query.set_insert_columns(COLUMNS_ENTRY_FILES)
            query.set_select_columns(COLUMNS_GIT_ENTRY_FILES)
            query.set_from_tables([TABLE_GIT_ENTRY_FILES])
            self.connection.execute(str(query))

        self.log.log(self.engine_name, "Finished to load backup metadata", LOG_LEVEL_INFO)


import datetime

# METADATA - Entry Tables
class EntryOrder:

    def __init__(self, cod_entity_target: str, cod_entity_src:str, column_name: str,num_branch: int, order_type: str, owner: str):
        self.cod_entity_target = cod_entity_target
        self.cod_entity_src = cod_entity_src
        self.column_name = column_name
        self.num_branch = num_branch
        self.order_type = order_type
        self.owner = owner

    def get(self):
        return [self.cod_entity_target,self.cod_entity_src, self.column_name, self.num_branch, self.order_type, self.owner]

    def __str__(self):
        return "'"+self.cod_entity_target+"','"+self.cod_entity_src+"','"+self.column_name.replace("'","''")+"',"+str(self.num_branch)+",'"+self.order_type+"','"+self.owner+"'"


class EntryAggregators:

    def __init__(self, cod_entity_target: str, cod_entity_src:str, column_name: str, num_branch: int, owner: str):
        self.cod_entity_target = cod_entity_target
        self.cod_entity_src = cod_entity_src
        self.column_name = column_name
        self.num_branch = num_branch
        self.owner = owner

    def get(self):
        return [self.cod_entity_target,self.cod_entity_src, self.column_name, self.num_branch, self.owner]

    def __str__(self):
        return "'"+self.cod_entity_target+"','"+self.cod_entity_src+"','"+self.column_name.replace("'","''")+"',"+str(self.num_branch)+",'"+self.owner+"'"


class EntryFilters:

    def __init__(self, cod_entity_target: str, value: str, num_branch: int, owner: str):
        self.cod_entity_target = cod_entity_target
        self.value = value
        self.num_branch = num_branch
        self.owner = owner

    def get(self):
        return [self.cod_entity_target, self.value, self.num_branch, self.owner]

    def __str__(self):
        return "'"+self.cod_entity_target+"','"+self.value.replace("'","''")+"',"+str(self.num_branch)+",'"+self.owner+"'"

class EntryHaving:

    def __init__(self, cod_entity_target: str, value: str, num_branch: int, owner: str):
        self.cod_entity_target = cod_entity_target
        self.value = value
        self.num_branch = num_branch
        self.owner = owner

    def get(self):
        return [self.cod_entity_target, self.value, self.num_branch, self.owner]

    def __str__(self):
        return "'"+self.cod_entity_target+"','"+self.value.replace("'","''")+"',"+str(self.num_branch)+",'"+self.owner+"'"


class EntryDatasetRelationships:

    def __init__(self, cod_entity_master: str, column_name_master: str, cod_entity_detail: str, column_name_detail: str, relationship_type: str, owner: str):
        self.cod_entity_master = cod_entity_master
        self.column_name_master = column_name_master
        self.cod_entity_detail = cod_entity_detail
        self.column_name_detail = column_name_detail
        self.relationship_type = relationship_type
        self.owner = owner

    def get(self):
        return [self.cod_entity_master, self.column_name_master, self.cod_entity_detail, self.column_name_detail, self.relationship_type, self.owner]

    def __str__(self):
        return "'"+self.cod_entity_master+"','"+self.column_name_master.replace("'","''")+"','"+self.cod_entity_detail+"','"+self.column_name_detail.replace("'","''")+"','"+self.relationship_type+"','"+self.owner+"'"


class EntryEntity:

    def __init__(self, cod_entity: str, table_name: str, entity_type: str, cod_path: str, strategy: str, owner: str):
        self.cod_entity = cod_entity
        self.table_name = table_name
        self.entity_type = entity_type
        self.cod_path = cod_path
        self.strategy = strategy
        self.owner = owner

    def get(self):
        return [self.cod_entity, self.table_name, self.entity_type, self.cod_path, self.strategy, self.owner]

    def __str__(self):
        return "'"+self.cod_entity+"','"+self.table_name+"','"+self.entity_type+"','"+self.cod_path+"','"+self.strategy+"','"+self.owner+"'"


class EntryPath:

    def __init__(self, cod_path: str, database_name: str, schema_name: str, owner: str):
        self.cod_path = cod_path
        self.database_name = database_name
        self.schema_name = schema_name
        self.owner = owner

    def get(self):
        return [self.cod_path, self.database_name, self.schema_name, self.owner]

    def __str__(self):
        database_name = 'NULL'
        if self.database_name is not None and self.database_name != '': database_name = "'"+self.database_name + "'"
        schema_name = 'NULL'
        if self.schema_name is not None and self.schema_name != '': schema_name = "'" + self.schema_name + "'"
        return "'"+self.cod_path+"'," + database_name+","+schema_name+",'"+self.owner + "'"

class EntryFiles:

    def __init__(self, cod_entity: str, file_path: str, file_name: str, delimiter_character: str, owner: str):
        self.cod_entity = cod_entity
        self.file_path = file_path
        self.file_name = file_name
        self.delimiter_character = delimiter_character
        self.owner = owner

    def get(self):
        return [self.cod_entity, self.file_path, self.file_name, self.delimiter_character, self.owner]

    def __str__(self):
        delimiter_character = 'NULL'
        if self.delimiter_character is not None and self.delimiter_character != '': delimiter_character = "'"+self.delimiter_character + "'"
        return "'"+self.cod_entity+"','" + self.file_path + "','" + self.file_name+"',"+delimiter_character+",'"+self.owner + "'"


class EntryDatasetMappings:

    def __init__(self, cod_entity_source: str, value_source: str, cod_entity_target: str, column_name_target: str, column_type_target: str, ordinal_position: int, column_length : int, column_precision: int, num_branch: int, key_type: str, sw_distinct: int, owner: str):
        self.cod_entity_source = cod_entity_source
        self.value_source = value_source
        self.cod_entity_target = cod_entity_target
        self.column_name_target = column_name_target
        self.column_type_target = column_type_target
        self.ordinal_position = ordinal_position
        self.column_length = column_length
        self.column_precision = column_precision
        self.num_branch = num_branch
        self.key_type = key_type
        self.sw_distinct = sw_distinct
        self.owner = owner

    def get(self):
        return [self.cod_entity_source, self.value_source, self.cod_entity_target, self.column_name_target,
                self.column_type_target, self.ordinal_position, self.column_length, self.column_precision, self.num_branch,
                self.key_type, self.sw_distinct, self.owner]

    def __str__(self):
        key_type = 'NULL'
        if self.key_type is not None and self.key_type != '': key_type = "'"+self.key_type+"'"
        return "'"+self.cod_entity_source+"','"+self.value_source.replace("'","''")+"','"+self.cod_entity_target+"','" + self.column_name_target+"','"+self.column_type_target+"',"+str(self.ordinal_position) + "," + str(self.column_length)+","+str(self.column_precision)+","+str(self.num_branch)+","+key_type+","+str(self.sw_distinct)+",'"+self.owner+"'"

class EntryDvMappings:

    def __init__(self, cod_entity_source, column_name_source, cod_entity_target,column_name_target, column_type_target, ordinal_position, column_length, column_precision, num_branch,num_connection, key_type, satellite_name, origin_is_incremental, origin_is_total, origin_is_cdc, owner):
        self.cod_entity_source = cod_entity_source
        self.column_name_source = column_name_source
        self.cod_entity_target = cod_entity_target
        self.column_name_target = column_name_target
        self.column_type_target = column_type_target
        self.ordinal_position = ordinal_position
        self.column_length = column_length
        self.column_precision = column_precision
        self.num_branch = num_branch
        self.num_connection = num_connection
        self.key_type = key_type
        self.satellite_name = satellite_name
        self.origin_is_incremental = origin_is_incremental
        self.origin_is_total = origin_is_total
        self.origin_is_cdc = origin_is_cdc
        self.owner = owner

    def get(self):
        return [self.cod_entity_source, self.column_name_source, self.cod_entity_target, self.column_name_target,self.column_type_target,self.ordinal_position,self.column_length, self.column_precision,self.num_branch, self.num_connection, self.key_type,self.satellite_name,self.origin_is_incremental, self.origin_is_total, self.origin_is_cdc, self.owner]

    def __str__(self):
        return "'" + self.cod_entity_source +"','"+self.column_name_source.replace("'","''")+"','"+self.cod_entity_target+"','"+self.column_name_target+"','"+self.column_type_target+"',"+str(self.ordinal_position)+","+str(self.column_length)+\
                ","+str(self.column_precision) + ","+ str(self.num_branch)+","+str(self.num_connection)+",'"+self.key_type+"','"+self.satellite_name+"',"+str(self.origin_is_incremental)+","+str(self.origin_is_total)+","+str(self.origin_is_cdc)+",'"+self.owner+"'"

class EntryDvEntry:

    def __init__(self, cod_entity, entity_name, entity_type, cod_path, name_status_tracking_satellite, name_record_tracking_satellite,
                name_effectivity_satellite, owner):
        self.cod_entity = cod_entity
        self.entity_name = entity_name
        self.entity_type = entity_type
        self.cod_path = cod_path
        self.name_status_tracking_satellite = name_status_tracking_satellite
        self.name_record_tracking_satellite = name_record_tracking_satellite
        self.name_effectivity_satellite = name_effectivity_satellite
        self.owner = owner

    def get(self):
        return[self.cod_entity, self.entity_name, self.entity_type, self.cod_path, self.name_status_tracking_satellite,self.name_record_tracking_satellite,self.name_effectivity_satellite, self.owner]

    def __str__(self):
        return "'"+self.cod_entity+"','"+self.entity_name+"','"+self.entity_type+"','"+self.cod_path+"','"+self.name_status_tracking_satellite+"','"+self.name_record_tracking_satellite+"',"+ \
               "'"+self.name_effectivity_satellite+"','"+self.owner+"'"

class EntryDvProperties:

    def __init__(self, cod_entity, num_connection, hash_name, owner):
        self.cod_entity = cod_entity
        self.num_connection = num_connection
        self.hash_name = hash_name
        self.owner = owner

    def get(self):
        return [self.cod_entity, self.num_connection, self.hash_name, self.owner]

    def __str__(self):
        return "'" + self.cod_entity + "'," + self.num_connection + ",'" + self.hash_name + "','" + self.owner + "'"

# METADATA - Metamorf Tables
class OmDataset:

    def __init__(self, id_dataset, dataset_name, id_entity_type, id_path, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_dataset = id_dataset
        self.dataset_name = dataset_name
        self.id_entity_type = id_entity_type
        self.id_path = id_path
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_dataset, self.dataset_name, self.id_entity_type, self.id_path, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_dataset) + ",'" + self.dataset_name + "'," + str(self.id_entity_type) + "," + str(self.id_path) + ",'" + self.meta_owner + "'," + self.start_date + "," + end_date

class OmDatasetDv:

    def __init__(self, id_dataset, id_entity_type, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_dataset =  id_dataset
        self.id_entity_type = id_entity_type
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_dataset, self.id_entity_type, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_dataset) + ","+ str(self.id_entity_type) +",'"+self.meta_owner +"'," + self.start_date + "," + end_date

class OmDatasetSpecification:

    def __init__(self, id_dataset_spec, id_dataset, id_key_type, column_name, column_type, ordinal_position, is_nullable, column_length, column_precision, column_scale, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_dataset_spec = id_dataset_spec
        self.id_dataset = id_dataset
        self.id_key_type = id_key_type
        self.column_name = column_name
        self.column_type = column_type
        self.ordinal_position = ordinal_position
        self.is_nullable = is_nullable
        self.column_length = column_length
        self.column_precision = column_precision
        self.column_scale = column_scale
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_dataset_spec, self.id_dataset, self.id_key_type, self.column_name, self.column_type, self.ordinal_position, self.is_nullable, self.column_length, self.column_precision, self.column_scale, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_dataset_spec) + ","+ str(self.id_dataset)+","+str(self.id_key_type)+",'"+str(self.column_name)+"','"+str(self.column_type)+"',"+str(self.ordinal_position)+","+str(self.is_nullable)+","+str(self.column_length)+","+str(self.column_precision)+","+str(self.column_scale)+",'"+str(self.meta_owner) + "'," + self.start_date + "," + end_date


class OmDatasetTAgg:

    def __init__(self, id_t_agg, id_dataset, id_branch, id_dataset_spec, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_t_agg = id_t_agg
        self.id_dataset = id_dataset
        self.id_branch = id_branch
        self.id_dataset_spec = id_dataset_spec
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_t_agg, self.id_dataset, self.id_branch, self.id_dataset_spec, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_t_agg) + "," + str(self.id_dataset) + "," + str(self.id_branch) + ","+ str(self.id_dataset_spec) + ",'" + self.meta_owner + "'," + self.start_date + "," + end_date


class OmDatasetTOrder:

    def __init__(self, id_t_order, id_dataset, id_branch, id_dataset_spec, id_order_type, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_t_order = id_t_order
        self.id_dataset = id_dataset
        self.id_branch = id_branch
        self.id_dataset_spec = id_dataset_spec
        self.id_order_type = id_order_type
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_t_order, self.id_dataset, self.id_branch, self.id_dataset_spec, self.id_order_type, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_t_order)+","+str(self.id_dataset)+","+str(self.id_branch)+","+str(self.id_dataset_spec)+","+str(self.id_order_type)+",'"+self.meta_owner+"',"+self.start_date+","+end_date


class OmDatasetTDistinct:

    def __init__(self, id_t_distinct, id_dataset, id_branch, sw_distinct, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_t_distinct = id_t_distinct
        self.id_dataset = id_dataset
        self.id_branch = id_branch
        self.sw_distinct = sw_distinct
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_t_distinct, self.id_dataset, self.id_branch, self.sw_distinct, self.meta_owner,  self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_t_distinct)+","+str(self.id_dataset)+","+str(self.id_branch)+","+str(self.sw_distinct)+",'"+str(self.meta_owner)+"',"+self.start_date+","+end_date


class OmDatasetExecution:

    def __init__(self, id_dataset, id_query_type, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_dataset = id_dataset
        self.id_query_type = id_query_type
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_dataset, self.id_query_type, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_dataset)+","+str(self.id_query_type)+",'"+self.meta_owner+"',"+self.start_date+","+end_date


class OmProperties:

    def __init__(self, property, value, start_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        self.property = property
        self.value = value
        self.start_date = start_date

    def get(self):
        return [self.property, self.value, self.start_date]

    def __str__(self):
        return "'"+self.property+"','"+self.value+"',"+self.start_date+""


class OmDatasetRelationships:

    def __init__(self, id_relationship, id_dataset_spec_master, id_dataset_spec_detail, id_join_type, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_relationship = id_relationship
        self.id_dataset_spec_master = id_dataset_spec_master
        self.id_dataset_spec_detail = id_dataset_spec_detail
        self.id_join_type = id_join_type
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_relationship, self.id_dataset_spec_master, self.id_dataset_spec_detail, self.id_join_type, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_relationship)+","+str(self.id_dataset_spec_master)+","+str(self.id_dataset_spec_detail)+","+str(self.id_join_type)+",'"+self.meta_owner+"',"+self.start_date+","+end_date


class OmDatasetTMapping:

    def __init__(self, id_t_mapping, id_branch, id_dataset_spec, value_mapping, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_t_mapping = id_t_mapping
        self.id_branch = id_branch
        self.id_dataset_spec = id_dataset_spec
        self.value_mapping = value_mapping
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_t_mapping, self.id_branch, self.id_dataset_spec, self.value_mapping, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_t_mapping)+","+str(self.id_branch)+","+str(self.id_dataset_spec)+",'"+self.value_mapping.replace("'","''")+"','"+self.meta_owner+"',"+self.start_date+","+end_date


class OmDatasetPath:

    def __init__(self, id_path, database_name, schema_name, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_path = id_path
        self.database_name = database_name
        self.schema_name = schema_name
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_path, self.database_name, self.schema_name, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.database_name != None and self.database_name != '': database = "'"+self.database_name+"'"
        else: database = "NULL"
        if self.schema_name != None and self.schema_name != '': schema = "'"+self.schema_name+"'"
        else: schema = "NULL"
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_path)+","+database+","+schema+",'"+self.meta_owner+"',"+self.start_date+","+end_date

class OmDatasetFile:

    def __init__(self, id_dataset, file_path, file_name, delimiter_character, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_dataset = id_dataset
        self.file_path = file_path
        self.file_name = file_name
        self.delimiter_character = delimiter_character
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_dataset, self.file_path ,self.file_name ,self.delimiter_character ,self.meta_owner ,self.start_date ,self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_dataset)+",'"+str(self.file_path)+"','"+self.file_name+"','"+str(self.delimiter_character)+"','"+self.meta_owner+"',"+self.start_date +","+end_date


class OmDatasetTFilter:

    def __init__(self, id_t_filter, id_dataset, id_branch, value_filter, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_t_filter = id_t_filter
        self.id_dataset = id_dataset
        self.id_branch = id_branch
        self.value_filter = value_filter
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_t_filter, self.id_dataset, self.id_branch, self.value_filter, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_t_filter)+","+str(self.id_dataset)+","+str(self.id_branch)+",'"+self.value_filter.replace("'","''")+"','"+self.meta_owner+"',"+self.start_date+","+end_date


class OmDatasetTHaving:

    def __init__(self, id_t_having, id_dataset, id_branch, value_having, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_t_having = id_t_having
        self.id_dataset = id_dataset
        self.id_branch = id_branch
        self.value_having = value_having
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_t_having, self.id_dataset, self.id_branch, self.value_having, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_t_having)+","+str(self.id_dataset)+","+str(self.id_branch)+",'"+self.value_having+"','"+self.meta_owner+"',"+self.start_date+","+end_date


class OmDatasetHardcoded:

    def __init__(self, id_dataset_hardcoded, id_dataset, id_branch, content, meta_owner, start_date, end_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d %H:%M:%S")
        if isinstance(end_date, datetime.datetime): end_date = end_date.strftime("%Y-%m-%d %H:%M:%S")
        self.id_dataset_hardcoded = id_dataset_hardcoded
        self.id_dataset = id_dataset
        self.id_branch = id_branch
        self.content = content
        self.meta_owner = meta_owner
        self.start_date = start_date
        self.end_date = end_date

    def get(self):
        return [self.id_dataset_hardcoded, self.id_dataset, self.id_branch, self.content, self.meta_owner, self.start_date, self.end_date]

    def __str__(self):
        if self.end_date is None or self.end_date == '' or self.end_date=='NULL': end_date = "NULL"
        else: end_date = self.end_date
        return str(self.id_dataset_hardcoded)+","+str(self.id_dataset)+","+str(self.id_branch)+",'"+self.content+"','"+self.meta_owner+"',"+self.start_date+","+end_date

        # METADATA - Reference Tables
class OmRefQueryType:

    def __init__(self, id_query_type, query_type_name, query_type_description):
        self.id_query_type = id_query_type
        self.query_type_name = query_type_name
        self.query_type_description = query_type_description


class OmRefOrderType:

    def __init__(self, id_order_type, order_type_name, order_type_value, order_type_description):
        self.id_order_type = id_order_type
        self.order_type_name = order_type_name
        self.order_type_value = order_type_value
        self.order_type_description = order_type_description


class OmRefJoinType:

    def __init__(self, id_join_type, join_name, join_value, join_description):
        self.id_join_type = id_join_type
        self.join_name = join_name
        self.join_value = join_value
        self.join_description = join_description


class OmRefKeyType:

    def __init__(self, id_key_type, key_type_name, key_type_description):
        self.id_key_type = id_key_type
        self.key_type_name = key_type_name
        self.key_type_description = key_type_description


class OmRefEntityType:

    def __init__(self, id_entity_type, entity_type_name, entity_type_description, entity_type_full_name, id_module):
        self.id_entity_type = id_entity_type
        self.entity_type_name = entity_type_name
        self.entity_type_description = entity_type_description
        self.entity_type_full_name = entity_type_full_name
        self.id_module = id_module

class OmRefModules:

    def __init__(self, id_module, module_name, module_full_name, module_description):
        self.id_module = id_module
        self.module_name = module_name
        self.module_full_name = module_full_name
        self.module_description = module_description

# IM Tables
class OmDatasetInformation:

    def __init__(self, meta_owner, entity_type_full_name, entity_type_description, module_name, module_full_name, module_description, dataset_name, database_name, schema_name, query_type_name, query_type_description, start_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d, %H:%M:%S")
        self.meta_owner = meta_owner
        self.entity_type_full_name = entity_type_full_name
        self.entity_type_description = entity_type_description
        self.module_name = module_name
        self.module_full_name = module_full_name
        self.module_description = module_description
        self.dataset_name = dataset_name
        self.database_name = database_name
        self.schema_name = schema_name
        self.query_type_name = query_type_name
        self.query_type_description = query_type_description
        self.start_date = start_date

class OmRelationships:

    def __init__(self, meta_owner, master_database_name, master_schema_name, master_dataset_name, master_column_name, detail_database_name, detail_schema_name, detail_dataset_name, detail_column_name, start_date):
        if isinstance(start_date, datetime.datetime): start_date = start_date.strftime("%Y-%m-%d, %H:%M:%S")
        self.meta_owner = meta_owner
        self.master_database_name = master_database_name
        self.master_schema_name = master_schema_name
        self.master_dataset_name = master_dataset_name
        self.master_column_name = master_column_name
        self.detail_database_name = detail_database_name
        self.detail_schema_name = detail_schema_name
        self.detail_dataset_name = detail_dataset_name
        self.detail_column_name = detail_column_name
        self.start_date = start_date

class OmDatasetSpecificationInformation:

    def __init__(self, meta_owner, dataset_name, column_name, column_type, ordinal_position, is_nullable, column_length, column_precision, column_scale):
        self.meta_owner = meta_owner
        self.dataset_name = dataset_name
        self.column_name = column_name
        self.column_type = column_type
        self.ordinal_position = ordinal_position
        self.is_nullable = is_nullable
        self.column_length = column_length
        self.column_precision = column_precision
        self.column_scale = column_scale


class Column:

    def __init__(self, id, column_name, column_type, default_value, key_type, is_nullable, column_length, column_precision, column_scale):
        self.id = id
        self.column_name = column_name
        self.column_type = column_type
        self.default_value = default_value
        self.key_type = key_type
        self.is_nullable = is_nullable
        self.column_length = column_length
        self.column_precision = column_precision
        self.column_scale = column_scale

    def __str__(self):
        return str(self.id) + "," + str(self.column_name) +"," + str(self.column_type) +"," + str(self.default_value) +"," + str(self.key_type) + "," + self.is_nullable + "," + self.column_length + "," + self.column_precision + "," + self.column_scale







